#ifndef jpegH
#define jpegH
#include <math.h>
#include <extctrls.hpp>

const int tonoGris=0x010101;

template <class Ctipo>Ctipo** nuevaMatriz(int n, int m)
   {
      Ctipo **aux;
      aux=(Ctipo**)malloc(n*sizeof(Ctipo*));
      for(int i=0;i<n;i++)
         aux[i]=(Ctipo*)malloc(m*sizeof(Ctipo));
      return aux;
   }

template <class Ctipo>void liberarMatriz(Ctipo **M, int n, int m=0)
   {
      for(int i=0;i<n;i++)
         free(M[i]);
      free(M);
   }

template <class Ctipo>Ctipo mayor(Ctipo **A, int M, int N)
   {
      Ctipo actual=A[0][0];
      for(int i=0;i<M;i++)
         for(int j=0;j<N;j++)
            if(A[i][j]>actual)
               actual=A[i][j];
      return actual;
   }

template <class Ctipo>Ctipo menor(Ctipo **A, int M, int N)
   {
      Ctipo actual=A[0][0];
      for(int i=0;i<M;i++)
         for(int j=0;j<N;j++)
            if(A[i][j]<actual)
               actual=A[i][j];
      return actual;
   }

void normalizar(double **A, int M, int N)
   {
      double Max=mayor(A,M,N), Min=menor(A,M,N);
      double dif=Max-Min;
      for(int i=0;i<M;i++)
         for(int j=0;j<N;j++)
            A[i][j]=(A[i][j]-Min)/dif;
   }

double alpha(double x, double N)
   {
      return (x==0)?1.0/sqrt(N):sqrt(2.0/N);
   }

double **TDC(double **I, int M, int N)
   {
      double **aux=nuevaMatriz<double>(M,N);
      
      for(int k=0;k<M;k++)
         for(int l=0;l<N;l++)
            {
               double suma=0;
               
               for(int m=0;m<M;m++)
                  for(int n=0;n<N;n++)
                     suma+=I[m][n]*cos((M_PI*(2.0*m+1.0)*k)/(2.0*M))*cos((M_PI*(2.0*n+1.0)*l)/(2.0*N));
               aux[k][l]=suma*alpha(k,M)*alpha(l,N);
            }
      return aux;
   }

double **TDCs(double **I, int M, int N)
   {
      double **aux=nuevaMatriz<double>(M,N);
      double **aux2=nuevaMatriz<double>(M,N);
      
      for(int k=0;k<M;k++)//fijamos una fila y...
         for(int l=0;l<N;l++)
            {
               double suma=0;
               for(int n=0;n<N;n++)
                  suma+=I[k][n]*cos((M_PI*(2.0*n+1.0)*l)/(2.0*M));
               aux[k][l]=suma;
            }
            
      for(int l=0;l<N;l++)
         for(int k=0;k<M;k++)
            {
               double suma=0;
               for(int m=0;m<N;m++)
                  suma+=aux[m][l]*cos((M_PI*(2.0*m+1.0)*k)/(2.0*N));
               aux2[k][l]=suma*alpha(l,N)*alpha(k,M);
            }
      liberarMatriz<double>(aux, M, N);
      return aux2;
   }


double **TDCi(double **I, int M, int N)
   {
      double **aux=nuevaMatriz<double>(M,N);

      for(int m=0;m<M;m++)
         for(int n=0;n<N;n++)
            {
               double suma=0;

               for(int k=0;k<M;k++)
                  for(int l=0;l<N;l++)
                     suma+=I[k][l]*alpha(k,M)*alpha(l,N)*cos((M_PI*(2.0*m+1.0)*k)/(2.0*M))*cos((M_PI*(2.0*n+1.0)*l)/(2.0*N));
               aux[m][n]=suma;
            }
      return aux;
   }

double **TDCis(double **I, int M, int N)
   {
      double **aux=nuevaMatriz<double>(M,N);
      double **aux2=nuevaMatriz<double>(M,N);
      
      for(int m=0;m<M;m++)//fijamos una fila y...
         for(int n=0;n<N;n++)
            {
               double suma=0;
               for(int l=0;l<N;l++)
                  suma+=I[m][l]*cos((M_PI*(2.0*n+1.0)*l)/(2.0*N))*alpha(l,N);
               aux[m][n]=suma;
            }
            
      for(int n=0;n<N;n++)
         for(int m=0;m<M;m++)
            {
               double suma=0;
               for(int k=0;k<M;k++)
                  suma+=aux[k][n]*cos((M_PI*(2.0*m+1.0)*k)/(2.0*M))*alpha(k,M);
               aux2[m][n]=suma;
            }
      liberarMatriz<double>(aux, M, N);
      return aux2;
   }


void copiarImagen(TImage *I, double **img)
   {
      for (int i=0;i<I->Height;i++)
         for(int j=0;j<I->Width;j++)
            img[i][j]=((int)(I->Canvas->Pixels[j][i]))%256;
   }

void mostrarImagen(double **img, TImage *I)
   {
      double Max=mayor<double>(img, I->Height, I->Width), Min=menor<double>(img, I->Height, I->Width), dif=Max-Min;
      
      for (int i=0;i<I->Height;i++)
         for(int j=0;j<I->Width;j++)
            I->Canvas->Pixels[j][i]=(TColor)((int)(((img[i][j]-Min)/dif)*255)*tonoGris);
      I->Repaint();
   }

void sigPosZigZag(int &f, int &c, int &dir, int M, int N)
   {
      if(dir==0)//va para arriba  a la derecha
         {
            if(f==0)
               {
                  if(c==N-1)
                     {
                        f++;
                        dir=1;//abajo-izquierda
                     }
                  else
                     {
                        c++;
                        dir=1;
                     }
               }
            else if(c==N-1)
               {
                  f++;
                  dir=1;
               }
            else
               {
                  f--;
                  c++;                  
               }
         }
      else//va para abajo a la izquierda
         {
               if(c==0)
                  {
                     if(f==M-1)
                         {
                            c++;
                            dir=0;
                         }
                      else
                         {
                            f++;
                            dir=0;
                         }
                  }
               else if(f==M-1)
                  {
                     c++;
                     dir=0;
                  }
               else
                  {
                     c--;
                     f++;
                  }
         }
   }
   
double **diferenciarEnZigZag(double **A, int M, int N)
   {
      int dir=0;//arriba-derecha
      int f=0, c=0, fAnt=0, cAnt=0;
      double **aux=nuevaMatriz<double>(M,N);
      aux[fAnt][cAnt]=A[fAnt][cAnt];
      while(f!=M-1 || c!=N-1)
         {
            sigPosZigZag(f,c,dir,M,N);
            aux[f][c]=A[fAnt][cAnt]-A[f][c];
            fAnt=f;
            cAnt=c;
         }
      return aux;
   }

void multiplicar(double **A, int M,int N, double x)
   {
      for(int i=0;i<M;i++)
         for(int j=0;j<N;j++)
            A[i][j]*=x;
   }

double entropia(double **A, int M,int N)
   {
      double prob[256], prod=M*N, suma=0;
      normalizar(A,M,N);
      multiplicar(A,M,N,255);
      for(int i=0;i<256;i++)
         prob[i]=0;


      for(int i=0;i<M;i++)
         for(int j=0;j<N;j++)         
            prob[(int)A[i][j]]+=1;
      
      for(int k=0;k<256;k++)
         if (prob[k]>0)
            {
               prob[k]/=prod;
               suma+=prob[k]*log(prob[k]);
            }
      return -1.0/log(2.0)*suma;
   }
   
#endif